#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#define F_CPU 8000000                   // CPU Takt in Hz
#define FRAMERATE 100                   // Bildwiederholrate fr LED-Matrix

#define DATA_SET    PORTB |=  (1<<4)    // serielle Daten fuer Scheiberegister
#define DATA_CLR    PORTB &= ~(1<<4)    // 
#define CLK_SET     PORTB |=  (1<<3)    // Takt fuer Schieberegister
#define CLK_CLR     PORTB &= ~(1<<3)
#define STROBE_SET  PORTB |=  (1<<2)    // speichern der seriellen Daten im Ausgangsregister
#define STROBE_CLR  PORTB &= ~(1<<2)

// Kodierung der LEDs fr die Ziffern 0-9, Balkenanzeige
const uint16_t hex2led[10] = {
    0b0000000000,
    0b1000000000,
    0b1100000000,
    0b1110000000,
    0b1111000000,
    0b1111100000,
    0b1111110000,
    0b1111111000,
    0b1111111100,
    0b1111111110
};

// Variablen

volatile uint16_t led[12];

volatile unsigned char sekunden;
volatile unsigned char minuten;
volatile unsigned char stunden;
volatile unsigned char flag_1s;

void Anzeige()
{ 
    led[2] = hex2led[stunden/10];
    led[3] = hex2led[stunden%10];

    led[5] = hex2led[minuten/10];
    led[6] = hex2led[minuten%10];

    led[8] = hex2led[sekunden/10];
    led[9] = hex2led[sekunden%10];
}

void inittimer()
{
    // Timer 2 ,32768 Hz, 8Bit, Prescaler 128 -> 1s
    /*
    ASSR|=(1<<AS2);                // Asynchronmodus
    TCCR2=(1<<CS22) | (1<<CS20);   // Prescaler 128
    TIMSK|=(1<<TOIE2);             // enable Interrupt
*/
    // Timer1, mode 4, CTC, Prescaler=1 
    TCCR1B = (1<<WGM12) | (1<<CS10);
    TIMSK  |= (1<<OCIE1A);              // OCR1A Interrupt enable
    OCR1A = (F_CPU / (12 * FRAMERATE))-1;
}

// Timer 2, 1s

ISR(TIMER2_OVF_vect)
{
    flag_1s=1;
    sekunden++;
    if(sekunden==60)
    {
        sekunden=0;
        minuten++;
        if(minuten==60)
        {
            minuten=0;
            stunden++;
            if(stunden==24)
            {
                stunden=0;
            }
        }
    }
}

// Timer 1, 830us fr LED-Multiplexing

ISR(TIMER1_COMPA_vect)
{
    uint16_t spalte=0;
    static uint8_t count=0;
    static uint8_t cnt_1s=0;

    // Spaltenmultiplexing ber Schieberegister
    count++;
    if (count==12)
    {
        count=0;
        DATA_CLR;
        cnt_1s++;
        if (cnt_1s == FRAMERATE) {
            cnt_1s = 0;
            flag_1s=1;            
        }
    }
    else
    {
        DATA_SET;
    }
    
    // altes Zeilenmuster lschen
    PORTB &= ~0x03;
    PORTC &= ~0x0F; 
    PORTD &= ~0xF0;

    // neues Muster lesen
    spalte=led[count];
    
    // Spalten-Mux einen Takt weiter schieben
    CLK_SET;
    CLK_CLR;

    // neue Spalte einschalten           
    STROBE_SET;
    STROBE_CLR;

    // neues Muster ausgeben
    PORTB |=(spalte >> 8) & 0x03;
    PORTC |=spalte & 0x0F; 
    PORTD |=spalte & 0xF0;

} 

int main(void)
{  
    sekunden=50;
    minuten=47;
    stunden=13;
 
    DDRC = 0x0f; // PC0-PC3 (R1-R9) zu Ausgnge
    DDRB = 0xff; // PD4-PD7 (R5-R8) zu Ausgnge
    DDRD = 0xff; // PB0-PB1 (R9-R10() zu Ausgnge

    //ACSR = 0x80;

    PORTD = 0;   // alle Bits auf 0
    PORTB = 0; 
    PORTC = 0;

    inittimer();

    sei();

    while(1)
    {
        set_sleep_mode(SLEEP_MODE_IDLE);
        sleep_mode();

        if (flag_1s) {
            flag_1s = 0;
            sekunden++;
            if(sekunden==60)
            {
                sekunden=0;
                minuten++;
                if(minuten==60)
                {
                    minuten=0;
                    stunden++;
                    if(stunden==24)
                    {
                        stunden=0;
                    }
                }
            }
            Anzeige();
        }
    }
}

